'This script accompanies the book Windows Management Instrumentation
'by Matthew Lavy and Ashley Meggitt (New Riders, 2001)
'The code is copyright 2001, Matthew Lavy & Ashley Meggitt
'You are free to use and modify the script at will
'provided you understand that all code here is made available AS IS, with
'NO WARRANTY WHATSOEVER. The authors take NO RESPONSIBILITY AT ALL for
'problems that result from the use of this script or any part thereof

'changeownerandperm.vbs -- change the owner of
'a specified file or directory on a specified machine
'or change the permissions that a specified user has over
'that file or directory
Option Explicit
On Error Resume Next
Dim refWMI
Dim refSetting
Dim refSD
Dim strMachine
Dim strObject
Dim strPerson
Dim strAction
Dim numSuccess
If WScript.arguments.count <> 4 Then
	WScript.Echo _
		"Usage: changeownerandperms.vbs <machine> <object> <person> <action>"
	WScript.Echo " where <action> is OWN, FULL, READ, or REMOVE"
	WScript.quit
End If

'assign arguments to named variables for clarity later on
strMachine = WScript.arguments(0)
strObject = WScript.arguments(1)
strPerson = WScript.arguments(2)
strAction = WScript.arguments(3)

'connect to WMI
set refWMI = GetObject("winMgmts:{(Security,Restore)}!\\" & strMachine)
If Err <> 0 Then
	WScript.Echo "Could not connect to machine " & strMachine
End If

'find a LogicalFileSecuritySetting
Set refSetting = refWMI.Get("Win32_LogicalFileSecuritySetting='" & _
		strObject & "'")
If Err <> 0 Then
	WScript.Echo "Cannot find a LogicalFileSecuritySetting for " & strObject
	set refWMI = Nothing
	WScript.quit
End If

'get security descriptor
If refSetting.GetSecurityDescriptor(refSD) <> 0 Then
	WScript.Echo "Cannot retrieve the security descriptor for " & strObject
End If

'work out what to do
Select Case strAction
Case "OWN":
	WScript.Echo "Attempting to make " & strPerson & _
		" owner of " & strObject
	numSuccess = makeOwner(strPerson, refSD, refWMI)

Case "FULL":
	WScript.Echo "Attempting to give Full Control of " & _
		strObject & " to " & strPerson
	numSuccess = changeDACL(strPerson, refSD, refWMI,2032127)

Case "READ":
	WScript.Echo "Attempting to give Read Access of " & _
		strObject & " to " & strPerson
	numSuccess = changeDACL(strPerson, refSD, refWMI,131245)

Case "REMOVE":
	WScript.Echo "Attempting to remove " & strPerson & _
		" from the DACL of " & strObject
	numSuccess = changeDACL(strPerson, refSD, refWMI,0)

Case Else:
	WScript.Echo "Action must be one of: OWN, FULL, READ, REMOVE"
	set refSD = Nothing
	set refSetting = Nothing
	set refWMI = Nothing
	WScript.quit
End Select

If numSuccess <> 0 Then
	WScript.Echo "Failed to manipulate security descriptor."
Else
	If refSetting.SetSecurityDescriptor(refSD) <> 0 Then
		WScript.Echo "Failed to re-write Security Descriptor."
	Else
		WScript.Echo "Operation successful!"
	End If
End If
set refSD = Nothing
Set refSetting = Nothing
Set refWMI = Nothing

'--- END OF MAIN SCRIPT ---

Function changeDACL(strPerson, refSD, refWMI,numPerms)
	On Error Resume Next
	Dim refDictionary
	Dim refItem
	Dim refTrustee
	Dim refACE
	'create a dictionary and populate it with DACL
	'index by Trustee.Name
	set refDictionary = CreateObject("Scripting.Dictionary")
	For Each refItem In refSD.DACL
		refDictionary.add UCase(refItem.Trustee.Name), refItem
	Next
	
	'First deal with case when ACE should be removed
	If numPerms = 0 Then
		'if there is an ACE corresponding to the person
		'delete it from dictionary
		If refDictionary.Exists(UCase(strPerson)) Then
			refDictionary.Remove(UCase(strPerson))
			refSD.DACL = refDictionary.Items
		End If
	
		'that's all we need to do
		ChangeDACL = Err
		set refDictionary = Nothing
		Exit Function
	End If

	'If we got here, we need an ACE. So if there isn't one
	'already, we must create it
	If Not refDictionary.Exists(UCase(strPerson)) Then
		'don't check return value here because it'll be caught
		createTrustee strPerson,refWMI,refTrustee
		
		'create ACE
		set refACE = refWMI.Get("Win32_ACE").SpawnInstance_()
		With refACE
			.AceType = 0
			.AceFlags = 3
			.Trustee = refTrustee
		End With

		'did it work?!
		If Err <> 0 Then
			ChangeDACL = Err
			set refTrustee = Nothing
			set refACE = Nothing
			set refDictionary = Nothing
			Exit Function
		End If

		'cool - it worked. Add it to the dictionary
		refDictionary.add UCase(strPerson),refACE
	Else
		'we have an ACE, so get a ref to it
		set refACE = refDictionary.item(UCase(strPerson))
	End If

	'now alter the access mask and copy back to DACL
	refACE.AccessMask = numPerms
	refSD.DACL = refDictionary.Items
	changeDACL = Err
	set refACE = Nothing
	set refTrustee = Nothing
End Function

Function makeOwner(strPerson,refSD,refWMI)
	On Error Resume Next
	Dim refTrustee
	Dim numError
	'create a Trustee or die
	numError = createTrustee(strPerson,refWMI,refTrustee)
	If numError <> 0 Then
		makeOwner = numError
		Exit Function
	End If

	'set owner
	set refSD.Owner = refTrustee
End Function

Function createTrustee(strPerson,refWMI,refTrustee)
	On Error Resume Next
	Dim colWinAcc
	Dim refSID
	Dim refItem
	'find a SID
	set colWinAcc = refWMI.ExecQuery("SELECT * FROM Win32_ACCOUNT " & _
		"WHERE Name='" & strPerson & "'")
	For Each refItem in colWinAcc
		set refSID = refWMI.Get("Win32_SID='" & refItem.SID & "'")
	Next

	'create trustee
	Set refTrustee = refWMI.Get("Win32_Trustee").spawnInstance_()
	With refTrustee
		.Domain = refSID.ReferencedDomainName
		.Name = refSID.AccountName
		.SID = refSID.BinaryRepresentation
		.SidLength = refSID.SidLength
		.SIDString = refSID.SID
	End With
	createTrustee = Err
End Function
