'This script accompanies the book Windows Management Instrumentation
'by Matthew Lavy and Ashley Meggitt (New Riders, 2001)
'The code is copyright 2001, Matthew Lavy & Ashley Meggitt
'You are free to use and modify the script at will
'provided you understand that all code here is made available AS IS, with
'NO WARRANTY WHATSOEVER. The authors take NO RESPONSIBILITY AT ALL for
'problems that result from the use of this script or any part thereof

'monitorprintserver.vbs - set up permanent event subscription to monitor
'a print server and its printers
Option Explicit
Dim refWMI
Dim refFilterClass
Dim refFilter1
Dim refFilter2
Dim refConsumer
Dim refBindingClass
Dim refBinding1
Dim refBinding2

'connect to WMI
set refWMI = GetObject("winMgmts:")

'create filter objects and configure
'as we are making two instances, we may as well
'get a reference to the class first rather than
'doing it all in one step
set refFilterClass = refWMI.Get("__EventFilter")
set refFilter1 = refFilterClass.SpawnInstance_()
set refFilter2 = refFilterClass.SpawnInstance_()

'filter to monitor printer itself
'note the query that checks every 2 minutes
'and matches only where the Win32_Printer object detects
'an error state
With refFilter1
	.Name = "PrinterTroubleFilter"
	.QueryLanguage = "WQL"
	.Query = "SELECT * FROM __InstanceModificationEvent WITHIN 120 " & _
		"WHERE TargetInstance ISA 'Win32_Printer' AND " & _
		"TargetInstance.DetectedError <> 0"
End With
refFilter1.Put_()

'filter to detect changes to status of print spooler
'check is made once every two minutes on the print
'spooler service only
With refFilter2
	.Name = "SpoolerChangeFilter"
	.QueryLanguage = "WQL"
	.Query = "SELECT * FROM __InstanceModificationEvent WITHIN 120 " & _
		"WHERE TargetInstance ISA 'Win32_Service' AND " & _
		"TargetInstance.Name = 'spooler'"
End With
refFilter2.Put_()

'Now create and configure the consumer
'The message contains enough information to uniquely
'identify the type of event that occurred
'and the TargetInstance involved. This information comes
'from the __CLASS system property of the __Event
'and the __PATH system property of the TargetInstance
set refConsumer = refWMI.Get("SMTPEventConsumer").SpawnInstance_()
With refConsumer
	.Name = "MailW2KAdministrator"
	.SMTPServer = "smtp.myorganisation.com"
	.FromLine = "%TargetInstance.__SERVER%"
	.ToLine = "w2kadmin@myorganisation.com"
	.Subject = "PROBLEM WITH PRINT SERVER"
	.Message = "An %__CLASS% was delivered concerning " & _
		"%TargetInstance.__PATH%"
End With
refConsumer.Put_()

'create two __FilterToConsumerBinding instances
set refBindingClass = refWMI.Get("__FilterToConsumerBinding")
set refBinding1 = refBindingClass.SpawnInstance_()
set refBinding2 = refBindingClass.SpawnInstance_()

'bind the filters with consumers. Note that we use
'the RelPath property of SWbemObjectPath to ensure
'that a careless typo in specifying the names
'of __EventFilters and SMTPEventConsumer does not
'prevent the subscription from operating
With refBinding1
	.Filter = refFilter1.Path_.RelPath
	.Consumer = refConsumer.Path_.RelPath
End With
refBinding1.Put_()

With refBinding2
	.Filter = refFilter2.Path_.RelPath
	.Consumer = refConsumer.Path_.RelPath
End With
refBinding2.Put_()

'assuming all of this worked without raising an error,
'permanent event subscription is now set up. So let's
'tidy up and go home!
set refBinding1 = Nothing
set refBinding2 = Nothing
set refBindingClass = Nothing
set refFilter1 = Nothing
set refFilter2 = Nothing
set refFilterClass = Nothing
set refConsumer = Nothing
set refWMI = Nothing
